var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var cmdm = require('../lib/tfcommand');
var buildifm = require('vso-node-api/interfaces/BuildInterfaces');
var params = require('../lib/parameternames');
var os = require('os');
function describe() {
    return 'queue a build';
}
exports.describe = describe;
function getArguments() {
    return cmdm.formatArgumentsHint([params.PROJECT_NAME], [params.DEFINITION_ID, params.DEFINITION_NAME]) + os.EOL + '\tmust supply ' + params.DEFINITION_ID + ' or ' + params.DEFINITION_NAME;
}
exports.getArguments = getArguments;
function getCommand() {
    // this just offers description for help and to offer sub commands
    return new BuildQueue();
}
exports.getCommand = getCommand;
// requires auth, connect etc...
exports.isServerOperation = true;
// unless you have a good reason, should not hide
exports.hideBanner = false;
var BuildQueue = (function (_super) {
    __extends(BuildQueue, _super);
    function BuildQueue() {
        _super.apply(this, arguments);
    }
    BuildQueue.prototype.exec = function (args, options) {
        var _this = this;
        var buildapi = this.getWebApi().getQBuildApi();
        var project = args[0] || options[params.PROJECT_NAME];
        this.checkRequiredParameter(project, params.PROJECT_NAME, params.PROJECT_FRIENDLY_NAME);
        var definitionId = +args[1] || +options[params.DEFINITION_ID];
        if (definitionId) {
            return buildapi.getDefinition(definitionId, project).then(function (definition) {
                return _this._queueBuild(buildapi, definition, project);
            });
        }
        else {
            var definitionName = options[params.DEFINITION_NAME];
            this.checkRequiredParameter(definitionName, params.DEFINITION_NAME);
            return buildapi.getDefinitions(project, definitionName).then(function (definitions) {
                var definition = definitions[0];
                return _this._queueBuild(buildapi, definition, project);
            });
        }
    };
    BuildQueue.prototype.output = function (data) {
        if (!data) {
            throw new Error('no build supplied');
        }
        var build = data;
        console.log();
        console.log('Queued new build:');
        console.log('id   : ' + build.id);
        console.log('definition name: ' + build.definition.name);
        console.log('requested by : ' + build.requestedBy.displayName);
        console.log('status : ' + buildifm.BuildStatus[build.status]);
        console.log('queue time : ' + build.queueTime);
    };
    BuildQueue.prototype._queueBuild = function (buildapi, definition, project) {
        var build = {
            definition: definition
        };
        return buildapi.queueBuild(build, project);
    };
    return BuildQueue;
})(cmdm.TfCommand);
exports.BuildQueue = BuildQueue;
