/// <reference path="../../definitions/q/Q.d.ts"/>
/// <reference path="../../definitions/node/node.d.ts"/>
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var cmdm = require('../lib/tfcommand');
var vm = require('../lib/jsonvalidate');
var Q = require('q');
var path = require('path');
var params = require('../lib/parameternames');
var archiver = require('archiver');
function describe() {
    return 'upload a build task';
}
exports.describe = describe;
function getArguments() {
    return cmdm.formatArgumentsHint([params.TASK_PATH], []);
}
exports.getArguments = getArguments;
function getCommand() {
    return new BuildTaskUpload();
}
exports.getCommand = getCommand;
// requires auth, connect etc...
exports.isServerOperation = true;
// unless you have a good reason, should not hide
exports.hideBanner = false;
var c_taskJsonFile = 'task.json';
var BuildTaskUpload = (function (_super) {
    __extends(BuildTaskUpload, _super);
    function BuildTaskUpload() {
        _super.apply(this, arguments);
    }
    BuildTaskUpload.prototype.exec = function (args, options) {
        var _this = this;
        var deferred = Q.defer();
        var taskPath = args[0] || options[params.TASK_PATH];
        this.checkRequiredParameter(taskPath, params.TASK_PATH, 'path to the task folder');
        try {
            vm.exists(taskPath, 'specified directory does not exist.');
        }
        catch (directoryError) {
            deferred.reject(directoryError);
        }
        //directory is good, check json
        vm.validate(path.join(taskPath, c_taskJsonFile), 'no ' + c_taskJsonFile + ' in specified directory').then(function (taskJson) {
            var archive = archiver('zip');
            archive.on('error', function (error) {
                error.message = "Archiving error: " + error.message;
                deferred.reject(error);
            });
            archive.directory(taskPath, false);
            var agentapi = _this.getWebApi().getTaskAgentApi(_this.connection.accountUrl);
            agentapi.uploadTaskDefinition(archive, null, "", taskJson.id, false, function (err, statusCode, task) {
                if (err) {
                    err.statusCode = statusCode;
                    deferred.reject(err);
                }
                else {
                    deferred.resolve({
                        sourceLocation: taskPath
                    });
                }
            });
            archive.finalize();
        }).fail(function (error) {
            deferred.reject(error);
        });
        return deferred.promise;
    };
    BuildTaskUpload.prototype.output = function (data) {
        console.log('task at: ' + data.sourceLocation + ' uploaded successfully!');
    };
    return BuildTaskUpload;
})(cmdm.TfCommand);
exports.BuildTaskUpload = BuildTaskUpload;
